@@ -3,13 +3,15 @@
 #include <libbonobo.h>
 #include "gnome-vfs-client.h"
 #include "gnome-vfs-cancellation-private.h"
+#include "gnome-vfs-volume-monitor-private.h"
+#include "gnome-vfs-volume-monitor-client.h"
 
 BONOBO_CLASS_BOILERPLATE_FULL(
 	GnomeVFSClient,
 	gnome_vfs_client,
 	GNOME_VFS_Client,
 	BonoboObject,
-	BONOBO_TYPE_OBJECT);
+	BONOBO_TYPE_OBJECT)
 
 
 struct _GnomeVFSClientPrivate {
@@ -65,6 +67,87 @@ gnome_vfs_client_monitor_callback (PortableServer_Servant _servant,
   /* DAEMON-TODO: monitor support */
 }
 
+static void
+gnome_vfs_client_volume_mounted (PortableServer_Servant _servant,
+				 const GNOME_VFS_Volume * corba_volume,
+				 CORBA_Environment * ev)
+{
+	GnomeVFSVolume *volume;
+	GnomeVFSVolumeMonitor *volume_monitor;
+
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+
+	volume = _gnome_vfs_volume_from_corba (corba_volume, volume_monitor);
+	_gnome_vfs_volume_monitor_mounted (volume_monitor, volume);
+	gnome_vfs_volume_unref (volume);
+}
+
+static void
+gnome_vfs_client_volume_unmounted (PortableServer_Servant _servant,
+				   const CORBA_long id, CORBA_Environment * ev)
+{
+	GnomeVFSVolume *volume;
+	GnomeVFSVolumeMonitor *volume_monitor;
+
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+
+	volume = gnome_vfs_volume_monitor_get_volume_by_id (volume_monitor, id);
+	if (volume != NULL) {
+		_gnome_vfs_volume_monitor_unmounted (volume_monitor, volume);
+		gnome_vfs_volume_unref (volume);
+	}
+}
+
+static void
+gnome_vfs_client_volume_pre_unmount (PortableServer_Servant _servant,
+				     const CORBA_long id, CORBA_Environment * ev)
+{
+	GnomeVFSVolume *volume;
+	GnomeVFSVolumeMonitor *volume_monitor;
+
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+
+	volume = gnome_vfs_volume_monitor_get_volume_by_id (volume_monitor, id);
+	if (volume != NULL) {
+		gnome_vfs_volume_monitor_emit_pre_unmount (volume_monitor,
+							   volume);
+		gnome_vfs_volume_unref (volume);
+	}
+}
+
+
+static void
+gnome_vfs_client_drive_connected (PortableServer_Servant _servant,
+				  const GNOME_VFS_Drive * corba_drive,
+				  CORBA_Environment * ev)
+{
+	GnomeVFSDrive *drive;
+	GnomeVFSVolumeMonitor *volume_monitor;
+
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+
+	drive = _gnome_vfs_drive_from_corba (corba_drive, volume_monitor);
+	_gnome_vfs_volume_monitor_connected (volume_monitor, drive);
+	gnome_vfs_drive_unref (drive);
+}
+
+static void
+gnome_vfs_client_drive_disconnected (PortableServer_Servant _servant,
+				     const CORBA_long id,
+				     CORBA_Environment * ev)
+{
+	GnomeVFSDrive *drive;
+	GnomeVFSVolumeMonitor *volume_monitor;
+
+	volume_monitor = gnome_vfs_get_volume_monitor ();
+
+	drive = gnome_vfs_volume_monitor_get_drive_by_id (volume_monitor, id);
+	if (drive != NULL) {
+		_gnome_vfs_volume_monitor_disconnected (volume_monitor, drive);
+		gnome_vfs_drive_unref (drive);
+	}
+}
+
 static void
 gnome_vfs_client_class_init (GnomeVFSClientClass *klass)
 {
@@ -72,6 +155,11 @@ gnome_vfs_client_class_init (GnomeVFSClientClass *klass)
 	POA_GNOME_VFS_Client__epv *epv = &klass->epv; 
 
 	epv->MonitorCallback = gnome_vfs_client_monitor_callback;
+	epv->VolumeMounted = gnome_vfs_client_volume_mounted;
+	epv->VolumeUnmounted = gnome_vfs_client_volume_unmounted;
+	epv->VolumePreUnmount = gnome_vfs_client_volume_pre_unmount;
+	epv->DriveConnected = gnome_vfs_client_drive_connected;
+	epv->DriveDisconnected = gnome_vfs_client_drive_disconnected;
 	
 	object_class->finalize = gnome_vfs_client_finalize;
 }
@@ -81,6 +169,8 @@ static void
 daemon_connection_broken (gpointer connection,
 			  GnomeVFSClient *client)
 {
+	GnomeVFSVolumeMonitor *volume_monitor;
+
 	/* This is run in an idle, so some code might run between the
 	 * connection going bork and this code running.
 	 */
@@ -100,6 +190,15 @@ daemon_connection_broken (gpointer connection,
 	 */
 
 	G_UNLOCK (the_client);
+
+	volume_monitor = _gnome_vfs_get_volume_monitor_internal (FALSE);
+	if (volume_monitor != NULL) {
+		/* We need to remove all the old volumes/drives since
+		 * their ids are not the same for the next daemon.
+		 */
+		_gnome_vfs_volume_monitor_client_daemon_died (GNOME_VFS_VOLUME_MONITOR_CLIENT (volume_monitor));
+	}
+
 }
 
 
@@ -251,7 +350,7 @@ _gnome_vfs_get_client (void)
 		/* All Client callback happens in idle */
 		idle_poa = bonobo_poa_get_threaded (ORBIT_THREAD_HINT_ALL_AT_IDLE);
 		the_client = g_object_new (GNOME_TYPE_VFS_CLIENT,
-					   "poa", idle_poa);
+					   "poa", idle_poa, NULL);
 		CORBA_Object_release ((CORBA_Object) idle_poa, NULL);
 	}
 	G_UNLOCK (the_client);
