@@ -98,18 +98,18 @@ gnome_vfs_format_file_size_for_display (GnomeVFSFileSize size)
 typedef enum {
 	UNSAFE_ALL        = 0x1,  /* Escape all unsafe characters   */
 	UNSAFE_ALLOW_PLUS = 0x2,  /* Allows '+'  */
-	UNSAFE_PATH       = 0x4,  /* Allows '/' and '?' and '&' and '='  */
-	UNSAFE_DOS_PATH   = 0x8,  /* Allows '/' and '?' and '&' and '=' and ':' */
+	UNSAFE_PATH       = 0x4,  /* Allows '/' and '&' and '='  */
+	UNSAFE_DOS_PATH   = 0x8,  /* Allows '/' and '&' and '=' and ':' */
 	UNSAFE_HOST       = 0x10, /* Allows '/' and ':' and '@' */
 	UNSAFE_SLASHES    = 0x20  /* Allows all characters except for '/' and '%' */
 } UnsafeCharacterSet;
 
 static const guchar acceptable[96] =
 { /*  X0   X1   X2   X3   X4   X5   X6   X7   X8   X9   XA   XB   XC   XD   XE   XF */
-  /*   !    "    #    $    %    &    '    (    )    *    +    ,         -    .    / */
+  /* spc    !    "    #    $    %    &    '    (    )    *    +    ,    -    .    / */
     0x00,0x3F,0x20,0x20,0x20,0x00,0x2C,0x3F,0x3F,0x3F,0x3F,0x22,0x20,0x3F,0x3F,0x1C, 
   /*   0    1    2    3    4    5    6    7    8    9    :    ;    <    =    >    ? */
-    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x38,0x20,0x20,0x2C,0x20,0x2C, 
+    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x38,0x20,0x20,0x2C,0x20,0x20, 
   /*   @    A    B    C    D    E    F    G    H    I    J    K    L    M    N    O */
     0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F, 
   /*   P    Q    R    S    T    U    V    W    X    Y    Z    [    \    ]    ^    _ */
@@ -264,7 +264,7 @@ gnome_vfs_escape_string (const gchar *string)
  * @path: string to be escaped.
  *
  * Escapes @path, replacing only special characters that would not
- * be found in paths (so '/', '&', '=', and '?' will not be escaped by
+ * be found in paths (so '/', '&', and '=' will not be escaped by
  * this function).
  *
  * Return value: a newly allocated string equivalent to @path but
@@ -308,8 +308,8 @@ gnome_vfs_escape_path_string (const gchar *path)
  * @path: string to be escaped.
  *
  * Escapes @path, replacing only special characters that would not
- * be found in paths or host name (so '/', '&', '=', ':', '@' 
- * and '?' will not be escaped by this function).
+ * be found in paths or host name (so '/', '&', '=', ':' and '@'
+ * will not be escaped by this function).
  *
  * Return value: a newly allocated string equivalent to @path but
  * with non-path/host characters escaped.
@@ -1964,6 +1964,7 @@ _gnome_vfs_uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
 	GnomeVFSURI *new_uri, *resolved_uri;
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult res;
+	char *escaped_symlink;
 	char *p;
 	int n_followed_symlinks;
 
@@ -2000,8 +2001,19 @@ _gnome_vfs_uri_resolve_all_symlinks_uri (GnomeVFSURI *uri,
 				gnome_vfs_uri_unref (new_uri);
 				goto out;
 			}
+			escaped_symlink = gnome_vfs_escape_path_string (info->symlink_name);
 			resolved_uri = gnome_vfs_uri_resolve_relative (new_uri,
-								       info->symlink_name);
+								       escaped_symlink);
+			g_free (escaped_symlink);
+			if (resolved_uri == NULL) {
+				gnome_vfs_uri_unref (new_uri);
+				/* FIXME this shouldn't happen. gnome_vfs_uri_resolve_relative
+				 * treats some relative URIs as absolute if they contain special
+				 * characters. We should escape them, or the relative URI recognition
+				 * should be improved */
+				break;
+			}
+
 			if (*p != 0) {
 				gnome_vfs_uri_unref (new_uri);
 
@@ -2058,6 +2070,69 @@ _gnome_vfs_uri_resolve_all_symlinks (const char *text_uri,
 	return res;
 }
 
+char *
+gnome_vfs_resolve_symlink (const char *path,
+			   const char *symlink)
+{
+	char *p, *filename;
+	char **strs;
+	int i, j, n;
+	GString *res_path;
+
+	p = strrchr (path, '/');
+	
+	if (symlink[0] == '/' || p == NULL) {
+		return g_strdup (symlink);
+	}
+
+	/* either use whole path as base (if it ends in '/'),
+	 * or chop its filename part */
+	p = g_strndup (path, p - path);
+		
+	filename = g_build_filename (p, symlink, NULL);
+	g_free (p);
+
+	strs = g_strsplit (filename, "/", -1);
+
+	g_free (filename);
+
+	n = g_strv_length (strs);
+
+	for (i = 0; i < n; i++) {
+		if (!strcmp (strs[i], "") ||
+		    !strcmp (strs[i], ".")) {
+			g_free (strs[i]);
+			strs[i] = NULL;
+		} else if (!strcmp (strs[i], "..")) {
+			g_free (strs[i]);
+			strs[i] = NULL;
+
+			for (j = i; strs[j] == NULL && j > 0; j--)
+				;
+
+			g_free (strs[j]);
+			strs[j] = NULL;
+		}
+	}
+
+	res_path = g_string_new (NULL);
+
+	for (i = 0; i < n; i++)
+		if (strs[i] != NULL) {
+			g_string_append_c (res_path, '/');
+			g_string_append (res_path, strs[i]);
+			g_free (strs[i]);
+		}
+
+	/* TODO also re-append '/' if the symlink ends in '/'? */ 
+	if (res_path->len == 0)
+		g_string_append_c (res_path, '/');
+
+	g_free (strs);
+
+	return g_string_free (res_path, FALSE);
+}
+
 gboolean 
 _gnome_vfs_uri_is_in_subdir (GnomeVFSURI *uri, GnomeVFSURI *dir)
 {
