@@ -513,7 +513,7 @@ PrependOneURIToList (const gchar *rel_path, GnomeVFSFileInfo *info,
 	PrependOneURIParams *params;
 
 	params = (PrependOneURIParams *)cast_to_params;
-	params->uri_list = g_list_prepend (params->uri_list, gnome_vfs_uri_append_string (
+	params->uri_list = g_list_prepend (params->uri_list, gnome_vfs_uri_append_path (
 		params->base_uri, rel_path));
 
 	if (recursing_will_loop) {
@@ -755,10 +755,9 @@ list_add_items_and_size (const GList *name_uri_list,
 	each_params.progress = progress;
 	each_params.result = GNOME_VFS_OK;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	return gnome_vfs_visit_list (name_uri_list, info_options,
@@ -805,10 +804,9 @@ directory_add_items_and_size (GnomeVFSURI *dir_uri,
 	each_params.progress = progress;
 	each_params.result = GNOME_VFS_OK;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	return gnome_vfs_directory_visit_uri (dir_uri, info_options,
@@ -868,18 +866,17 @@ handle_merged_name_conflict_visit (const gchar *rel_path,
 	GnomeVFSFileInfo *source_info;
 	GnomeVFSFileInfo *target_info;
 	GnomeVFSResult result;
-	gboolean replace, skip;
+	gboolean skip;
 
 	params = data;
 
-	replace = FALSE;
 	skip = FALSE;
 	result = GNOME_VFS_OK;
 	target_info = gnome_vfs_file_info_new ();
-	target_uri = gnome_vfs_uri_append_string (params->target_uri, rel_path);
+	target_uri = gnome_vfs_uri_append_path (params->target_uri, rel_path);
 	if (gnome_vfs_get_file_info_uri (target_uri, target_info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK) {
 		source_info = gnome_vfs_file_info_new ();
-		source_uri = gnome_vfs_uri_append_string (params->source_uri, rel_path);
+		source_uri = gnome_vfs_uri_append_path (params->source_uri, rel_path);
 		
 		if (gnome_vfs_get_file_info_uri (source_uri, source_info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK) {
 			if (target_info->type != GNOME_VFS_FILE_TYPE_DIRECTORY ||
@@ -939,10 +936,9 @@ handle_merged_directory_name_conflicts (GnomeVFSXferOptions xfer_options,
 	if (xfer_options & GNOME_VFS_XFER_SAMEFS)
 		visit_options |= GNOME_VFS_DIRECTORY_VISIT_SAMEFS;
 
+	info_options = GNOME_VFS_FILE_INFO_DEFAULT;
 	if (xfer_options & GNOME_VFS_XFER_FOLLOW_LINKS) {
-		info_options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
-	} else {
-		info_options = GNOME_VFS_FILE_INFO_DEFAULT;
+		info_options |= GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 	}
 
 	params.result = GNOME_VFS_OK;
@@ -1026,9 +1022,15 @@ handle_name_conflicts (GList **source_uri_list,
 		    gnome_vfs_get_file_info_uri (uri, target_info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK) {
 			progress_set_source_target_uris (progress, source_uri, uri);
 			
-			/* no error getting info -- file exists, ask what to do */
-			replace = handle_overwrite (&result, progress, error_mode,
-						    overwrite_mode, &replace, &skip);
+			/* Skip if we are trying to copy the file to itself */
+			if (gnome_vfs_uri_equal (uri, source_uri)) {
+				replace = FALSE;
+				skip = TRUE;
+			} else {
+				/* no error getting info -- file exists, ask what to do */
+				replace = handle_overwrite (&result, progress, error_mode,
+							    overwrite_mode, &replace, &skip);
+			}
 			
 			/* FIXME bugzilla.eazel.com 1207:
 			 * move items to Trash here
@@ -1201,7 +1203,8 @@ copy_file_data (GnomeVFSHandle *target_handle,
 		GnomeVFSProgressCallbackState *progress,
 		GnomeVFSXferOptions xfer_options,
 		GnomeVFSXferErrorMode *error_mode,
-		guint block_size,
+		guint source_block_size,
+		guint target_block_size,
 		gboolean *skip)
 {
 	GnomeVFSResult result;
@@ -1209,6 +1212,7 @@ copy_file_data (GnomeVFSHandle *target_handle,
 	const char *write_buffer;
 	GnomeVFSFileSize total_bytes_read;
 	gboolean forget_cache;
+	guint block_size;
 
 	*skip = FALSE;
 
@@ -1216,6 +1220,7 @@ copy_file_data (GnomeVFSHandle *target_handle,
 		return GNOME_VFS_ERROR_INTERRUPTED;
 	}
 
+	block_size = MAX (source_block_size, target_block_size);
 	buffer = g_malloc (block_size);
 	total_bytes_read = 0;
 
@@ -1322,6 +1327,8 @@ xfer_open_source (GnomeVFSHandle **source_handle,
 	GnomeVFSResult result;
 	gboolean retry;
 	
+        call_progress_often (progress, GNOME_VFS_XFER_PHASE_OPENSOURCE);
+
 	*skip = FALSE;
 	do {
 		retry = FALSE;
@@ -1438,7 +1445,8 @@ copy_symlink (GnomeVFSURI *source_uri,
 }
 
 static GnomeVFSResult
-copy_file (GnomeVFSFileInfo *info,  
+copy_file (GnomeVFSFileInfo *info,
+	   GnomeVFSFileInfo *target_dir_info,
 	   GnomeVFSURI *source_uri,
 	   GnomeVFSURI *target_uri,
 	   GnomeVFSXferOptions xfer_options,
@@ -1452,6 +1460,10 @@ copy_file (GnomeVFSFileInfo *info,
 
 	progress->progress_info->phase = GNOME_VFS_XFER_PHASE_OPENSOURCE;
 	progress->progress_info->bytes_copied = 0;
+        
+        progress->progress_info->file_size = info->size;
+        progress_set_source_target_uris (progress, source_uri, target_uri);
+
 	result = xfer_open_source (&source_handle, source_uri,
 				   progress, xfer_options,
 				   error_mode, skip);
@@ -1489,6 +1501,8 @@ copy_file (GnomeVFSFileInfo *info,
 					  */
 					 (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE && info->io_block_size > 0)
 					 ? info->io_block_size : 8192,
+					 (target_dir_info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE && target_dir_info->io_block_size > 0)
+					 ? target_dir_info->io_block_size : 8192,
 					 skip);
 	}
 
@@ -1635,7 +1649,8 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 				progress_set_source_target_uris (progress, source_uri, dest_uri);
 
 				if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
-					result = copy_file (info, source_uri, dest_uri, 
+					result = copy_file (info, target_dir_info,
+							    source_uri, dest_uri, 
 							    xfer_options, error_mode, overwrite_mode, 
 							    progress, &skip_child);
 				} else if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
@@ -1648,7 +1663,8 @@ copy_directory (GnomeVFSFileInfo *source_file_info,
 						result = gnome_vfs_get_file_info_uri (source_uri, symlink_target_info,
 										      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
 						if (result == GNOME_VFS_OK) 
-							result = copy_file (symlink_target_info, source_uri, dest_uri, 
+							result = copy_file (symlink_target_info, target_dir_info,
+									    source_uri, dest_uri, 
 									    xfer_options, error_mode, overwrite_mode, 
 									    progress, &skip_child);
 						gnome_vfs_file_info_unref (symlink_target_info);
@@ -1773,11 +1789,10 @@ copy_items (const GList *source_uri_list,
 			/* get target_dir URI and file info to take care of SGID */
 			target_dir_info = gnome_vfs_file_info_new ();
 			result = gnome_vfs_get_file_info_uri (target_dir_uri, target_dir_info,
-																						GNOME_VFS_FILE_INFO_DEFAULT);
+							      GNOME_VFS_FILE_INFO_DEFAULT);
 			if (result == GNOME_VFS_OK && GNOME_VFS_FILE_INFO_SGID(target_dir_info)) {
 				info->gid = target_dir_info->gid;
 			}
-			gnome_vfs_file_info_unref (target_dir_info);
 			result = GNOME_VFS_OK;
 
 			/* optionally keep trying until we hit a unique target name */
@@ -1802,7 +1817,8 @@ copy_items (const GList *source_uri_list,
 				
 				
 				if (info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
-					result = copy_file (info, source_uri, target_uri, 
+					result = copy_file (info, target_dir_info,
+							    source_uri, target_uri, 
 							    xfer_options, error_mode, 
 							    &overwrite_mode_abort, 
 							    progress, &skip);
@@ -1862,6 +1878,8 @@ copy_items (const GList *source_uri_list,
 				
 				/* try again with new uri */
 			}
+
+			gnome_vfs_file_info_unref (target_dir_info);
 		}
 
 		gnome_vfs_file_info_unref (info);
