@@ -27,13 +27,14 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-monitor-private.h>
 
+#include <glib/gi18n-lib.h>
+
 typedef enum {
 	COMPUTER_HOME_LINK,
 	COMPUTER_ROOT_LINK,
@@ -311,14 +312,12 @@ drive_connected (GnomeVFSVolumeMonitor *volume_monitor,
 	char *name;
 	
 	G_LOCK (root_dir);
-	if (gnome_vfs_drive_is_user_visible (drive)) {
-		file = computer_file_new (COMPUTER_DRIVE);
-		name = gnome_vfs_drive_get_display_name (drive);
-		file->file_name = build_file_name (name, ".drive");
-		g_free (name);
-		file->drive = gnome_vfs_drive_ref (drive);
-		computer_file_add (dir, file);
-	}
+	file = computer_file_new (COMPUTER_DRIVE);
+	name = gnome_vfs_drive_get_display_name (drive);
+	file->file_name = build_file_name (name, ".drive");
+	g_free (name);
+	file->drive = gnome_vfs_drive_ref (drive);
+	computer_file_add (dir, file);
 	G_UNLOCK (root_dir);
 }
 
@@ -369,14 +368,12 @@ fill_root (ComputerDir *dir)
 	
 	for (l = drives; l != NULL; l = l->next) {
 		drive = l->data;
-		if (gnome_vfs_drive_is_user_visible (drive)) {
-			file = computer_file_new (COMPUTER_DRIVE);
-			name = gnome_vfs_drive_get_display_name (drive);
-			file->file_name = build_file_name (name, ".drive");
-			g_free (name);
-			file->drive = gnome_vfs_drive_ref (drive);
-			computer_file_add (dir, file);
-		}
+		file = computer_file_new (COMPUTER_DRIVE);
+		name = gnome_vfs_drive_get_display_name (drive);
+		file->file_name = build_file_name (name, ".drive");
+		g_free (name);
+		file->drive = gnome_vfs_drive_ref (drive);
+		computer_file_add (dir, file);
 	}
 	
 	for (l = volumes; l != NULL; l = l->next) {
@@ -489,10 +486,13 @@ get_data_for_drive (GnomeVFSDrive *drive)
 	char *icon;
 	char *data;
 	char *tmp1, *tmp2;
-	GnomeVFSVolume *volume;
+	GList *volume_list;
+
+	volume_list = gnome_vfs_drive_get_mounted_volumes (drive);
+	if (volume_list != NULL) {
+		GnomeVFSVolume *volume;
+		volume = GNOME_VFS_VOLUME (volume_list->data);
 
-	volume = gnome_vfs_drive_get_mounted_volume (drive);
-	if (volume != NULL) {
 		uri = gnome_vfs_volume_get_activation_uri (volume);
 		tmp1 = gnome_vfs_drive_get_display_name (drive);
 		tmp2 = gnome_vfs_volume_get_display_name (volume);
@@ -529,7 +529,7 @@ get_data_for_drive (GnomeVFSDrive *drive)
 }
 
 static char *
-get_data_for_network ()
+get_data_for_network (void)
 {
 	char *data;
 
@@ -545,7 +545,7 @@ get_data_for_network ()
 }
 
 static char *
-get_data_for_home ()
+get_data_for_home (void)
 {
 	char *data;
 	char *uri;
@@ -565,7 +565,7 @@ get_data_for_home ()
 }
 
 static char *
-get_data_for_root ()
+get_data_for_root (void)
 {
 	char *data;
 
@@ -752,7 +752,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	FileHandle *file_handle;
 
