@@ -1,4 +1,10 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+
+/* 
+ * THIS METHOD IS OBSOLETE, IT HAS BEEN SUPERSEDED BY THE HTTP NEON METHOD
+ * IN http-neon-method.c AND http-proxy.c
+ */
+
 /* http-method.c - The HTTP method implementation for the GNOME Virtual File
    System.
 
@@ -556,7 +562,7 @@ struct _Header {
 };
 typedef struct _Header Header;
 
-static Header headers[] = {
+static const Header headers[] = {
 	{ "Content-Length", set_content_length },
 	{ "Content-Type", set_content_type },
 	{ "Last-Modified", set_last_modified },
@@ -1344,7 +1350,9 @@ connect_to_uri (
 	g_return_val_if_fail (toplevel_uri != NULL, GNOME_VFS_ERROR_INTERNAL);
 
 	if (!g_ascii_strcasecmp (gnome_vfs_uri_get_scheme (&toplevel_uri->uri), 
-				"https")) {
+				"https") ||
+	    !g_ascii_strcasecmp (gnome_vfs_uri_get_scheme (&toplevel_uri->uri), 
+				"davs")) {
 		if (!gnome_vfs_ssl_enabled ()) {
 			return GNOME_VFS_ERROR_NOT_SUPPORTED;
 		}
